@@ -17,10 +17,10 @@ from rasa.shared.constants import (
     DEFAULT_DATA_PATH,
     DEFAULT_RESULTS_PATH,
 )
-from rasa.core.test import FAILED_STORIES_FILE
 import rasa.shared.utils.validation as validation_utils
 import rasa.cli.utils
 import rasa.utils.common
+from rasa.shared.importers.importer import TrainingDataImporter
 
 logger = logging.getLogger(__name__)
 
@@ -69,15 +69,16 @@ def add_subparser(
 
 def run_core_test(args: argparse.Namespace) -> None:
     """Run core tests."""
-    from rasa.test import test_core_models_in_directory, test_core, test_core_models
+    from rasa.model_testing import (
+        test_core_models_in_directory,
+        test_core,
+        test_core_models,
+    )
+    from rasa.core.test import FAILED_STORIES_FILE
 
     stories = rasa.cli.utils.get_validated_path(
         args.stories, "stories", DEFAULT_DATA_PATH
     )
-    if args.e2e:
-        stories = rasa.shared.data.get_test_directory(stories)
-    else:
-        stories = rasa.shared.data.get_core_directory(stories)
 
     output = args.out or DEFAULT_RESULTS_PATH
     args.errors = not args.no_errors
@@ -99,17 +100,22 @@ def run_core_test(args: argparse.Namespace) -> None:
         )
 
         if args.evaluate_model_directory:
-            test_core_models_in_directory(args.model, stories, output)
+            test_core_models_in_directory(
+                args.model, stories, output, use_conversation_test_files=args.e2e
+            )
         else:
             test_core(
                 model=model_path,
                 stories=stories,
                 output=output,
                 additional_arguments=vars(args),
+                use_conversation_test_files=args.e2e,
             )
 
     else:
-        test_core_models(args.model, stories, output)
+        test_core_models(
+            args.model, stories, output, use_conversation_test_files=args.e2e
+        )
 
     rasa.shared.utils.cli.print_info(
         f"Failed stories written to '{os.path.join(output, FAILED_STORIES_FILE)}'"
@@ -144,10 +150,18 @@ async def run_nlu_test_async(
         no_errors: indicates if incorrect predictions should be written to a file
                    or not.
     """
-    from rasa.test import compare_nlu_models, perform_nlu_cross_validation, test_nlu
+    from rasa.model_testing import (
+        compare_nlu_models,
+        perform_nlu_cross_validation,
+        test_nlu,
+    )
+
+    data_path = rasa.cli.utils.get_validated_path(data_path, "nlu", DEFAULT_DATA_PATH)
+    test_data_importer = TrainingDataImporter.load_from_dict(
+        training_data_paths=[data_path]
+    )
+    nlu_data = await test_data_importer.get_nlu_data()
 
-    nlu_data = rasa.cli.utils.get_validated_path(data_path, "nlu", DEFAULT_DATA_PATH)
-    nlu_data = rasa.shared.data.get_nlu_directory(nlu_data)
     output = output_dir or DEFAULT_RESULTS_PATH
     all_args["errors"] = not no_errors
     rasa.shared.utils.io.create_directory(output)
@@ -176,7 +190,7 @@ async def run_nlu_test_async(
                 continue
         await compare_nlu_models(
             configs=config_files,
-            nlu=nlu_data,
+            test_data=nlu_data,
             output=output,
             runs=runs,
             exclusion_percentages=percentages,
@@ -192,7 +206,7 @@ async def run_nlu_test_async(
             models_path, "model", DEFAULT_MODELS_PATH
         )
 
-        await test_nlu(model_path, nlu_data, output, all_args)
+        await test_nlu(model_path, data_path, output, all_args)
 
 
 def run_nlu_test(args: argparse.Namespace) -> None:
@@ -216,7 +230,7 @@ def run_nlu_test(args: argparse.Namespace) -> None:
     )
 
 
-def test(args: argparse.Namespace):
+def test(args: argparse.Namespace) -> None:
     """Run end-to-end tests."""
     setattr(args, "e2e", True)
     run_core_test(args)
