@@ -651,6 +651,15 @@ class TrainingDataGenerator:
 
         end_trackers = []
         for event in events:
+            if (
+                isinstance(event, ActionExecuted)
+                and event.action_text
+                and event.action_text not in self.domain.action_texts
+            ):
+                rasa.shared.utils.cli.print_warning(
+                    f"Test story '{step.block_name}' in '{step.source_name}' contains the bot utterance "
+                    f"'{event.action_text}', which is not part of the training data / domain."
+                )
             for tracker in trackers:
                 if isinstance(
                     event, (ActionReverted, UserUtteranceReverted, Restarted)
