@@ -1,10 +1,10 @@
-from typing import List, Optional, Text, Tuple, Union
+from typing import Optional, Text, Tuple, Union
 import tensorflow as tf
 import tensorflow_addons as tfa
 from tensorflow.python.keras.utils import tf_utils
 from tensorflow.python.keras import backend as K
 import numpy as np
-from rasa.utils.tensorflow.layers import DenseWithSparseWeights
+from rasa.utils.tensorflow.layers import RandomlyConnectedDense
 
 
 # from https://www.tensorflow.org/tutorials/text/transformer
@@ -17,8 +17,8 @@ class MultiHeadAttention(tf.keras.layers.Layer):
         num_heads: Positive integer, number of heads
             to repeat the same attention structure.
         attention_dropout_rate: Float, dropout rate inside attention for training.
-        sparsity: Float between 0 and 1. Fraction of the `kernel`
-            weights to set to zero.
+        density: Approximate fraction of trainable weights (in
+            `RandomlyConnectedDense` layers).
         unidirectional: Boolean, use a unidirectional or bidirectional encoder.
         use_key_relative_position: Boolean, if 'True' use key
             relative embeddings in attention.
@@ -34,7 +34,7 @@ class MultiHeadAttention(tf.keras.layers.Layer):
         units: int,
         num_heads: int,
         attention_dropout_rate: float = 0.0,
-        sparsity: float = 0.8,
+        density: float = 0.2,
         unidirectional: bool = False,
         use_key_relative_position: bool = False,
         use_value_relative_position: bool = False,
@@ -63,21 +63,19 @@ class MultiHeadAttention(tf.keras.layers.Layer):
         self._depth = units // self.num_heads
 
         # process queries
-        self._query_dense_layer = DenseWithSparseWeights(
-            units=units, use_bias=False, sparsity=sparsity
+        self._query_dense_layer = RandomlyConnectedDense(
+            units=units, use_bias=False, density=density
         )
         # process keys
-        self._key_dense_layer = DenseWithSparseWeights(
-            units=units, use_bias=False, sparsity=sparsity
+        self._key_dense_layer = RandomlyConnectedDense(
+            units=units, use_bias=False, density=density
         )
         # process values
-        self._value_dense_layer = DenseWithSparseWeights(
-            units=units, use_bias=False, sparsity=sparsity
+        self._value_dense_layer = RandomlyConnectedDense(
+            units=units, use_bias=False, density=density
         )
         # process attention output
-        self._output_dense_layer = DenseWithSparseWeights(
-            units=units, sparsity=sparsity
-        )
+        self._output_dense_layer = RandomlyConnectedDense(units=units, density=density)
 
         self._create_relative_embeddings()
 
@@ -394,8 +392,7 @@ class TransformerEncoderLayer(tf.keras.layers.Layer):
         filter_units: Positive integer, output dim of the first ffn hidden layer.
         dropout_rate: Float between 0 and 1; fraction of the input units to drop.
         attention_dropout_rate: Float, dropout rate inside attention for training.
-        sparsity: Float between 0 and 1. Fraction of the `kernel`
-            weights to set to zero.
+        density: Fraction of trainable weights in `RandomlyConnectedDense` layers.
         unidirectional: Boolean, use a unidirectional or bidirectional encoder.
         use_key_relative_position: Boolean, if 'True' use key
             relative embeddings in attention.
@@ -413,7 +410,7 @@ class TransformerEncoderLayer(tf.keras.layers.Layer):
         filter_units: int,
         dropout_rate: float = 0.1,
         attention_dropout_rate: float = 0.0,
-        sparsity: float = 0.8,
+        density: float = 0.2,
         unidirectional: bool = False,
         use_key_relative_position: bool = False,
         use_value_relative_position: bool = False,
@@ -427,7 +424,7 @@ class TransformerEncoderLayer(tf.keras.layers.Layer):
             units,
             num_heads,
             attention_dropout_rate,
-            sparsity,
+            density,
             unidirectional,
             use_key_relative_position,
             use_value_relative_position,
@@ -438,12 +435,12 @@ class TransformerEncoderLayer(tf.keras.layers.Layer):
 
         self._ffn_layers = [
             tf.keras.layers.LayerNormalization(epsilon=1e-6),
-            DenseWithSparseWeights(
-                units=filter_units, activation=tfa.activations.gelu, sparsity=sparsity
+            RandomlyConnectedDense(
+                units=filter_units, activation=tfa.activations.gelu, density=density
             ),  # (batch_size, length, filter_units)
             tf.keras.layers.Dropout(dropout_rate),
-            DenseWithSparseWeights(
-                units=units, sparsity=sparsity
+            RandomlyConnectedDense(
+                units=units, density=density
             ),  # (batch_size, length, units)
             tf.keras.layers.Dropout(dropout_rate),
         ]
@@ -498,8 +495,8 @@ class TransformerEncoder(tf.keras.layers.Layer):
         reg_lambda: Float, regularization factor.
         dropout_rate: Float between 0 and 1; fraction of the input units to drop.
         attention_dropout_rate: Float, dropout rate inside attention for training.
-        sparsity: Float between 0 and 1. Fraction of the `kernel`
-            weights to set to zero.
+        density: Approximate fraction of trainable weights (in
+            `RandomlyConnectedDense` layers).
         unidirectional: Boolean, use a unidirectional or bidirectional encoder.
         use_key_relative_position: Boolean, if 'True' use key
             relative embeddings in attention.
@@ -520,7 +517,7 @@ class TransformerEncoder(tf.keras.layers.Layer):
         reg_lambda: float,
         dropout_rate: float = 0.1,
         attention_dropout_rate: float = 0.0,
-        sparsity: float = 0.8,
+        density: float = 0.2,
         unidirectional: bool = False,
         use_key_relative_position: bool = False,
         use_value_relative_position: bool = False,
@@ -534,8 +531,8 @@ class TransformerEncoder(tf.keras.layers.Layer):
         self.unidirectional = unidirectional
 
         l2_regularizer = tf.keras.regularizers.l2(reg_lambda)
-        self._embedding = DenseWithSparseWeights(
-            units=units, kernel_regularizer=l2_regularizer, sparsity=sparsity
+        self._embedding = RandomlyConnectedDense(
+            units=units, kernel_regularizer=l2_regularizer, density=density
         )
         # positional encoding helpers
         self._angles = self._get_angles()
@@ -551,7 +548,7 @@ class TransformerEncoder(tf.keras.layers.Layer):
                 filter_units,
                 dropout_rate,
                 attention_dropout_rate,
-                sparsity,
+                density,
                 unidirectional,
                 use_key_relative_position,
                 use_value_relative_position,
@@ -563,8 +560,8 @@ class TransformerEncoder(tf.keras.layers.Layer):
         self._layer_norm = tf.keras.layers.LayerNormalization(epsilon=1e-6)
 
     def _get_angles(self) -> np.ndarray:
-        i = np.arange(self.units)[np.newaxis, :]
-        return 1 / np.power(10000, (2 * (i // 2)) / np.float32(self.units))
+        array_2d = np.arange(self.units)[np.newaxis, :]
+        return 1 / np.power(10000, (2 * (array_2d // 2)) / np.float32(self.units))
 
     def _positional_encoding(self, max_position: tf.Tensor) -> tf.Tensor:
         max_position = tf.cast(max_position, dtype=tf.float32)
